.//============================================================================
.// File:      populate.arc
.//
.// Description:
.// This archetype file contains instance population definitions
.// for each object tagged as a static population.
.//
.// Notice:
.// (C) Copyright 1999-2001 TOYO Corporation All rights reserved.
.//
.//============================================================================
.//
.//============================================================================
.// *** Static population utilities:
.//============================================================================
.//
.//============================================================================
.// StaticManyRelationshipPopulateionBody
.//
.// <Abstract>
.//  This function defines object node list array for static instance
.//  populations having 'many' type relathionship. Here are;
.//    Input parameter condition:
.//      attr_MaxExtentSize >= attr_StaticExtentSize
.//    The array definition format:
.//      {previous_node, object}
.//        Where:
.//        previous_node:create a link to a previous node list array member
.//                     :NULL(0) means very top node of a link
.//        object       :specify object pool array member
.//    Note:
.//      The very last node of a link should be stored a participant
.//      object set attribute, like mc_C_R2{&T_B_C_R2_node1s[0], 0}
.//    Example:
.//      {&T_B_C_R2_node1s[0], 0}
.// <Input>
.//  obj:instance reference to an object
.//  rel:instance reference to a relationship
.// <Output>
.//  attr_MaxExtentSize:max size of an instance array for each object
.//  attr_StaticExtentSize:size of a static instance array for each object
.//  body:node list array
.// <Example>
.//  .function StaticManyRelationshipPopulateionBody
.//    .param inst_ref obj  .// O_OBJ
.//    .param inst_ref rel  .// R_REL
.//    .assign attr_MaxExtentSize = 0
.//    .assign attr_StaticExtentSize = 0
.//    .//
.//    .if ((obj.Key_Lett == "B") and (rel.Numb == 11))
.//      .assign attr_MaxExtentSize = 2
.//      .assign attr_StaticExtentSize = 2
.//    {0                   , (void *)&T_M_ObjectPool_s[0]},
.//    {&T_B_M_R11_node1s[0], (void *)&T_M_ObjectPool_s[1]}
.//    .elif ((obj.Key_Lett == "C") and (rel.Numb == 11))
.//      .assign attr_MaxExtentSize = 2
.//      .assign attr_StaticExtentSize = 2
.//    {0                   , (void *)&T_M_ObjectPool_s[0]},
.//    {&T_C_M_R11_node1s[0], (void *)&T_M_ObjectPool_s[1]}
.//    .else
.//      .assign attr_MaxExtentSize = 0
.//      .assign attr_StaticExtentSize = 0
.//    .end if
.//  .end function
.//============================================================================
.function StaticManyRelationshipPopulateionBody
  .param inst_ref obj  .// O_OBJ
  .param inst_ref rel  .// R_REL
  .assign attr_MaxExtentSize = 0
  .assign attr_StaticExtentSize = 0
  .//
  .if ( false )
  .elif ((obj.Key_Lett == "RP_SP") and (rel.Numb == 100))
    .assign attr_MaxExtentSize = 4
    .assign attr_StaticExtentSize = 4
  {0                         , (void *)&O_ST_ObjectPool_s[0]},
  {&O_RP_SP_ST_R100_node1s[0], (void *)&O_ST_ObjectPool_s[3]},
  {0                         , (void *)&O_ST_ObjectPool_s[1]},
  {&O_RP_SP_ST_R100_node1s[2], (void *)&O_ST_ObjectPool_s[2]}
  .elif ((obj.Key_Lett == "R_SP") and (rel.Numb == 100))
    .assign attr_MaxExtentSize = 4
    .assign attr_StaticExtentSize = 4
  {0                        , (void *)&O_ST_ObjectPool_s[0]},
  {&O_R_SP_ST_R100_node1s[0], (void *)&O_ST_ObjectPool_s[1]},
  {0                        , (void *)&O_ST_ObjectPool_s[2]},
  {&O_R_SP_ST_R100_node1s[2], (void *)&O_ST_ObjectPool_s[3]}
  .else
    .assign attr_MaxExtentSize = 0
    .assign attr_StaticExtentSize = 0
  .end if
.end function
.//
.//============================================================================
.// StaticInstancePopulateionBody
.//
.// <Abstract>
.//  This function defines instance array list for static instance
.//  populations.
.// <Input>
.//  obj:instance reference to an object
.// <Output>
.//  attr_StaticExtentSize:size of a static instance array for each object
.//  body:instance array list
.// <Example>
.//  .function StaticInstancePopulateionBody
.//    .param inst_ref obj  .// O_OBJ
.//    .assign attr_StaticExtentSize = 0
.//    .if (obj.Key_Lett == "T")
.//      .assign attr_StaticExtentSize = 1
.//      .// tubeID, power, current_state
.//    {1, 500, O_T_STATE_1}
.//    .elif (obj.Key_Lett == "L")
.//      .assign attr_StaticExtentSize = 1
.//      .// lightID, current_state
.//    {1, O_L_STATE_1}
.//    .elif (obj.Key_Lett == "")
.//      .assign attr_StaticExtentSize = 0
.//    .end if
.//  .end function
.//============================================================================
.function StaticInstancePopulateionBody
  .param inst_ref obj  .// O_OBJ
  .assign attr_StaticExtentSize = 0
  .if (obj.Key_Lett == "B")
    .assign attr_StaticExtentSize = 1 
  .///* uU[ID, , ^C}ID, current_state */
  .///* {0, 0, 0, O_B_STATE_1} */
  {1, 1, 0, O_B_STATE_1}
  .elif (obj.Key_Lett == "L")
    .assign attr_StaticExtentSize = 1 
  .///* CgID, current_state */
  .///* {0, O_L_STATE_1} */
  {1, O_L_STATE_1}
  .elif (obj.Key_Lett == "M")
    .assign attr_StaticExtentSize = 1 
  .///* j^ID, ^C}ID, j^Ԋu, oߎ, mc_V_R5, current_state */
  .///* {0, 0, 0, 0, &O_V_ObjectPool_s[0], O_M_STATE_1} */
  {1, 0, 1, 0, &O_V_ObjectPool_s[0], O_M_STATE_1}
  .elif (obj.Key_Lett == "RP_SP")
    .assign attr_StaticExtentSize = 2 
  .///* vZXdlID, vZX, , bg, mc_ST_R100 */
  .///* {0, "m_JP_50", 0, 0, &O_RP_SP_ST_R100_node1s[0], } */
  {1, " ", 3, 1000, &O_RP_SP_ST_R100_node1s[1], },
  {2, " dグ", 7, 500, &O_RP_SP_ST_R100_node1s[3], }
  .elif (obj.Key_Lett == "R_SP")
    .assign attr_StaticExtentSize = 2 
  .///* dlID, dl, , mc_ST_R100, mc_ST_R106 */
  .///* {0, "m_JP_45", 0, &O_R_SP_ST_R100_node1s[0], (O_ST_s *)&O_ST_ObjectPool_s[0], } */
  {1, " Ic", 10, &O_R_SP_ST_R100_node1s[1], &O_ST_ObjectPool_s[0]},
  {2, " ڋʏĂ", 10, &O_R_SP_ST_R100_node1s[3], &O_ST_ObjectPool_s[2]}
  .elif (obj.Key_Lett == "ST")
    .assign attr_StaticExtentSize = 4 
  .///* dlID, vZXdlID, vZX菇ID, mc_R_SP_R100, mc_RP_SP_R100, mc_ST_R105_Part, mc_ST_R105_Form */
  .///* {0, 0, 0, (O_R_SP_s *)&O_R_SP_ObjectPool_s[0], (O_RP_SP_s *)&O_RP_SP_ObjectPool_s[0], (O_ST_s *)&O_ST_ObjectPool_s[0], (O_ST_s *)&O_ST_ObjectPool_s[0], } */
  {1, 1, 0, &O_R_SP_ObjectPool_s[0], &O_RP_SP_ObjectPool_s[0], &O_ST_ObjectPool_s[1], 0},
  {1, 2, 1, &O_R_SP_ObjectPool_s[0], &O_RP_SP_ObjectPool_s[1], 0, &O_ST_ObjectPool_s[0]},
  {2, 2, 2, &O_R_SP_ObjectPool_s[1], &O_RP_SP_ObjectPool_s[1], &O_ST_ObjectPool_s[3], 0},
  {2, 1, 3, &O_R_SP_ObjectPool_s[1], &O_RP_SP_ObjectPool_s[0], 0, &O_ST_ObjectPool_s[2]}
  .elif (obj.Key_Lett == "T")
    .assign attr_StaticExtentSize = 2 
  .///* `[uID, bg, , ^C}ID, mc_V_R1, mc_R3_subtype, mc_R3_object_id, current_state */
  .///* {0, 0, 0, 0, &O_V_ObjectPool_s[0], (void *)&O_((subtype_key_lett))_ObjectPool_s[0], O_((subtype_key_lett))_OBJECT_ID, O_T_STATE_1} */
  {1, 500, 5, 0, 0, (void *)&O_TC_ObjectPool_s[0], O_TC_OBJECT_ID, O_T_STATE_1},
  {2, 0, 0, 0, &O_V_ObjectPool_s[0], (void *)&O_TR_ObjectPool_s[0], O_TR_OBJECT_ID, O_T_STATE_1}
  .elif (obj.Key_Lett == "TC")
    .assign attr_StaticExtentSize = 1 
  .///* `[uID, , mc_T_R3, current_state */
  .///* {0, 0, &O_T_ObjectPool_s[0], O_TC_STATE_1} */
  {1, 5, &O_T_ObjectPool_s[0], O_TC_STATE_1}
  .elif (obj.Key_Lett == "TR")
    .assign attr_StaticExtentSize = 1 
  .///* `[uID, mc_T_R3, mc_R_SP_R107, mc_ST_R108, current_state */
  .///* {0, &O_T_ObjectPool_s[0], (O_R_SP_s *)&O_R_SP_ObjectPool_s[0], (O_ST_s *)&O_ST_ObjectPool_s[0], O_TR_STATE_1} */
  {2, &O_T_ObjectPool_s[1], &O_R_SP_ObjectPool_s[0], &O_ST_ObjectPool_s[0], O_TR_STATE_1}
  .elif (obj.Key_Lett == "V")
    .assign attr_StaticExtentSize = 1 
  .///* I[uID, mc_T_R1, mc_L_R2, mc_B_R4, mc_M_R5, current_state */
  .///* {0, &O_T_ObjectPool_s[0], &O_L_ObjectPool_s[0], &O_B_ObjectPool_s[0], &O_M_ObjectPool_s[0], O_V_STATE_1} */
  {1, &O_T_ObjectPool_s[1], &O_L_ObjectPool_s[0], &O_B_ObjectPool_s[0], &O_M_ObjectPool_s[0], O_V_STATE_1}
  .elif (obj.Key_Lett == "")
    .assign attr_StaticExtentSize = 0
  .end if
.end function
.//
.//============================================================================
.// Do not add anything at the end of this file!
.//============================================================================
