.//============================================================================
.// File:  sys_functions.arc
.//
.// Description:
.// This archetype file contains system level user definable archetype
.// functions.
.//
.// Notice:
.// (C) Copyright 1999-2002 ROX Software, Inc.
.//     All rights reserved.
.// Enhancements provided by TOYO Corporation.
.// Modified for TOPPERS by RICOH CO,LTD. CC Development Section.
.//============================================================================
.//
.//
.//============================================================================
.// *** Main Task utilities:
.//============================================================================
.//
.//============================================================================
.// GetMainTaskEntryDeclaration
.//
.// <Abstract>
.//  This function defines a main task entry.
.// <Input>
.// <Output>
.//  body:main function or main task entry
.// <Example>
.//  .function GetMainTaskEntryDeclaration
.//  void mainTask ( int taskId )
.//  .end function
.//============================================================================
.function GetMainTaskEntryDeclaration
void
entry_task ( VP_INT exinf )
.end function
.//
.//============================================================================
.// GetMainTaskSleepRequest (Modified by RICOH)
.//
.// <Abstract>
.// This function request to sleep task, if this task has no event.
.// <Input>
.// <Output>
.// body:after dispatchs events in main function or main task entry.
.// <Example>
.// .function GetMainTaskSleepRequest
.//   if( DequeueOoaSelfEvent() == 0 && DequeueOoaNonSelfEvent() == 0)
.//       slp_tsk();
.// .end function
.//============================================================================
.function GetMainTaskSleepRequest
    if(wai_sem(SEM_MAIN) < E_OK){
        syslog_0(LOG_NOTICE, "Main Dispatch Semaphore Wait Error!");
    }
.end function
.//
.//============================================================================
.// GetMainTaskEntryReturn
.//
.// <Abstract>
.//  This function defines return statements in a main function or 
.//  a main task entry.  If the main function has a non-void return
.//  type, a return statement should be defined.
.// <Input>
.// <Output>
.//  body:very last statements in main function or main task entry 
.// <Example>
.//  .function GetMainTaskEntryReturn
.//    return 0;
.//  .end function
.//============================================================================
.function GetMainTaskEntryReturn
  /* not return value */
.end function
.//
.//
.//============================================================================
.// fixing extents to a base address
.//============================================================================
.//
.//============================================================================
.// UserFixExtentBaseBegin
.//
.// <Abstract>
.//  This function gets called at the point just before the storage
.//  for the class extent (array of structures) is defined.
.//  Return a value used to ORG the object instance storage in memory.
.// <Input>
.//  BASE_ADDRESS information extracted from object description        
.// <Output>
.//  pragma or ORG statement to fix memory address for following define
.// <Example>
.//  .invoke UserFixExtentBaseBegin( base_address )
.//============================================================================
.function UserFixExtentBaseBegin
  .param string base_address
  .//
  .assign attr_result = "/* ${base_address} */"
.end function
.//
.//============================================================================
.// UserFixExtentBaseEnd
.//
.// <Abstract>
.//  This function gets called at the point just after the storage
.//  for the class extent (array of structures) is defined.
.//  Return a value used to complete the ORG the object instance storage.
.// <Input>
.//  BASE_ADDRESS information extracted from object description        
.// <Output>
.//  end pragma or blank
.// <Example>
.//  .invoke UserFixExtentBaseEnd( base_address )
.//============================================================================
.function UserFixExtentBaseEnd
  .param string base_address
  .//
  .assign attr_result = "/* ${base_address} */"
.end function
.//
.//
.//============================================================================
.// enabling/disabling interrupts
.//============================================================================
.//
.//============================================================================
.// UserEnableInterrupts
.//
.// <Abstract>
.//  This archetype function returns a string which represents
.//  the target instruction for enabling interrupts.
.// <Input>
.//  none
.// <Output>
.//  enable interrupts statement (or nothing)
.// <Example>
.//  .invoke UserEnableInterrupts()
.//============================================================================
.function UserEnableInterrupts
  .//
  .assign attr_result = "/* enable interrupts */"
.end function
.//
.//============================================================================
.// UserDisableInterrupts
.//
.// <Abstract>
.//  This archetype function returns a string which represents
.//  the target instruction for disabling interrupts.
.// <Input>
.//  none
.// <Output>
.//  disable interrupts statement (or nothing)
.// <Example>
.//  .invoke UserDisableInterrupts()
.//============================================================================
.function UserDisableInterrupts
  .//
  .assign attr_result = "/* disable interrupts */"
.end function
.//
.//============================================================================
.// event mutex defintions
.//============================================================================
.//
.//============================================================================
.// EnterEventMutex (Modified by RICOH)
.//
.// <Abstract>
.// This archtype function entry event function mutex.
.// next event function lock until leave event mutex.
.// <Input>
.//  none
.// <Output>
.//  entry event mutex
.// <Example>
.//  .invoke EnterEventMutex()
.//============================================================================
.function EnterEventMutex
  .//
  .assign attr_result = "wai_sem(SEM_EVENT);	/* entry event mutex */"
.end function
.//
.//============================================================================
.// LeaveEventMutex (Modified by RICOH)
.//
.// <Abstract>
.// This archtype function leave event function mutex.
.// can entry next event function.
.// <Input>
.//  none
.// <Output>
.//  leave event mutex
.// <Example>
.//  .invoke LeaveEventMutex()
.//============================================================================
.function LeaveEventMutex
  .//
  .assign attr_result = "sig_sem(SEM_EVENT);	/* leave event mutex */"
.end function
.//
.//============================================================================
.// WakeUpMainDispatcher (Modified by RICOH)
.//
.// <Abstract>
.// This archtype function wake up main event dispatcher.
.// <Input>
.//  none
.// <Output>
.//  executes main event dispatcher.
.// <Example>
.//  .invoke WakeUpMainDispatcher()
.//============================================================================
.function WakeUpMainDispatcher
 sig_sem(SEM_MAIN);			/* Wake up main process */
.end function
.//
.//============================================================================
.// WakeUpMainDispatcherInterleave (Modified by RICOH)
.//
.// <Abstract>
.// This archtype function wake up main event dispatcher.
.// in interleave bridge.
.// <Input>
.//  none
.// <Output>
.//  executes main event dispatcher.
.// <Example>
.//  .invoke WakeUpMainDispatcherInterleave()
.//============================================================================
.function WakeUpMainDispatcherInterleave
 sig_sem(SEM_MAIN);			/* Wake up main process */
.end function
.//
.//============================================================================
.// Do not add anything at the end of this file!
.//============================================================================
