.//============================================================================
.// File: sys_evpool.arc
.//
.// Description:
.// This archetype file generates OOA event pool mananagement implementation
.// code.
.//
.// Notice:
.// (C) Copyright 1999-2002 ROX Software, Inc.
.//     All rights reserved.
.// Modified for TOPPERS by RICOH CO,LTD. CC Development Section.
.//
.// This document contains confidential and proprietary information and
.// property of ROX Software, Inc.  No part of this document may be
.// reproduced without the express written permission of ROX Software, Inc.
.//============================================================================
.//
.invoke arc_env = GET_ENV_VAR( "ROX_MC_ARC_DIR" )
.assign mc_archetypes = arc_env.result
.if ( mc_archetypes == "" )
  .print "\nERROR: Environment variable ROX_MC_ARC_DIR not set."
  .exit 100
.end if
.//
.include "${mc_archetypes}/names.arc"
.include "${mc_archetypes}/files.arc"
.include "${mc_archetypes}/types.arc"
.include "${mc_archetypes}/sys_util.arc"
.include "${mc_archetypes}/debug.arc"
.// Include system level user defined archetype functions.
.invoke system_colors_path = GetSystemLevelColoringFilesPath()
.invoke system_functions_file = GetSystemLevelUserFunctionsFile()
.include "${system_colors_path.result}/${system_functions_file.result}"
.//
.//============================================================================
.// CreateSystemEventPoolPrologue
.//
.//============================================================================
.function CreateSystemEventPoolPrologue
  .param boolean  gen_declaration
  .//
  .invoke file_name = GetSystemEventPoolFileName()
/*****************************************************************************
  .if ( gen_declaration )
    .invoke file_ext = GetHeaderFileExtension()
 * File:  ${file_name.result}.${file_ext.result}
  .else
    .invoke file_ext = GetSourceFileExtension()
 * File:  ${file_name.result}.${file_ext.result}
  .end if
 *
  .//
  .invoke copyright = GetCopyrightNotice()
${copyright.body}\
  .assign copyright.body = ""
 ****************************************************************************/

  .invoke file_ext = GetHeaderFileExtension()
  .if ( gen_declaration )
    .// *** Declarations section
    .//
    .invoke file_ext = GetHeaderFileExtension()
#ifndef $u{file_name.result}_$u{file_ext.result}
#define $u{file_name.result}_$u{file_ext.result}

#ifdef	__cplusplus
extern "C" {
#endif

  .else
    .// *** Definitions section
    .//

    .invoke file_name = GetSystemInitFileName()
#include "${file_name.result}.${file_ext.result}"
    .// *** Mechanism level includes
    .invoke generics_file = GetSystemMechanismsFileName()
    .invoke user_file = GetSystemUserCalloutFileName()
#include "${generics_file.result}.${file_ext.result}"
#include "${user_file.result}.${file_ext.result}"
    .select any system from instances of TE_SYS
    .select many dom_dlsp_set related by system->TE_DLSP[R2002]
    .//

/* Include header(s) of domain level events union. */
    .for each dom_dlsp in dom_dlsp_set
      .if ( (dom_dlsp.MaxNonSelfEvents != 0) or (dom_dlsp.MaxSelfEvents != 0) )
        .invoke file_name = GetDomainEventsFileNameFromString( dom_dlsp.RegisteredDomName )
        .invoke hdr_file_ext = GetHeaderFileExtension()
#include "${file_name.result}.${hdr_file_ext.result}"
      .end if
    .end for  .// dom_dlsp in dom_dlsp_set
    .//
  .end if  .// gen_declaration
.end function
.//
.//============================================================================
.// CreateSystemEventPoolFileEpilogue
.//
.//============================================================================
.function CreateSystemEventPoolFileEpilogue
  .param boolean  gen_declaration
  .//
  .if ( gen_declaration )
    .// *** Declarations section
    .invoke file_name = GetSystemEventPoolFileName()
    .invoke file_ext = GetHeaderFileExtension()

#ifdef	__cplusplus
}
#endif

#endif  /* $u{file_name.result}_$u{file_ext.result} */
  .else
    .// *** Definitions section
  .end if
.end function
.//
.//============================================================================
.// GetSystemEventUnionName
.//============================================================================
.function GetSystemEventUnionName
  .//
  .assign attr_result = "SystemOoaEvents_u"
  .assign attr_code_var_name = "aSG_ooa_event_pool"
.end function
.//
.//============================================================================
.// CreateSystemOoaEventPool
.//============================================================================
.function CreateSystemOoaEventPool
  .param boolean gen_declaration
  .//
  .invoke max_ooa_events = GetMaxOoaEventsDefineName()
  .//
  .select any system from instances of TE_SYS
  .select many dom_dlsp_set related by system->TE_DLSP[R2002]
  .//
  .// *** Declare a 'super union' of all domain level events union definitions.
  .invoke ooa_event_union = GetSystemEventUnionName()
  .invoke max_ooa_events = GetMaxOoaEventsDefineName()
  .invoke prefix = GetPrefix()

/*****************************************************************************
 * Structure: ${ooa_event_union.result}
 * 'Super-union' of all OOA events in the system. For translation patterns
 * which can not accept dynamic memory allocation for OOA events, this union
 * is used to predetermine the maximum size of any OOA event in the system.
 ****************************************************************************/
union ${ooa_event_union.result}
{
  .invoke event_base_class = GetOoaEventBaseClassName()
  ${event_base_class.result} ${event_base_class.var_name};
  .//
  .for each dom_dlsp in dom_dlsp_set
    .if ( (dom_dlsp.MaxNonSelfEvents != 0) or (dom_dlsp.MaxSelfEvents != 0) )
      .invoke dom_ooa_event_union = GetDomainEventsUnionNameFromString( dom_dlsp.RegisteredDomName )
  ${dom_ooa_event_union.result} mc_events_in_domain_${dom_dlsp.RegisteredDomName};
    .end if
  .end for
};
typedef union ${ooa_event_union.result} ${ooa_event_union.result};

/*****************************************************************************
 * Return the size of an OOA event.  Argument unused for fixed event size.
 ****************************************************************************/
unsigned
${prefix.symbol}GetOoaEventSize( const ${prefix.type}OoaEvent_t * const event )
{
  return sizeof( ${ooa_event_union.result} );
}

/*****************************************************************************
 * Provide anchor declaration for front and back of list of events.
 ****************************************************************************/
typedef struct OoaEventQueue_s OoaEventQueue_s;
struct OoaEventQueue_s
{
  ${prefix.type}OoaEvent_t * head, * tail;
};

/*****************************************************************************
 * Pre-allocated memory pool for OOA events.
 ****************************************************************************/
static ${ooa_event_union.result} ${ooa_event_union.code_var_name}[ ${max_ooa_events.result} ];
static ${prefix.type}OoaEvent_t * pSG_free_event_list = (${prefix.type}OoaEvent_t *) 0;
  .//

/*****************************************************************************
 * Link the event skeleton nodes together on the free list ready
 * for allocation.
 ****************************************************************************/
void
${prefix.symbol}InitializeOoaEventPool()
{
  unsigned int i;
  /* String events together into a singly linked list. */
  pSG_free_event_list = (${prefix.type}OoaEvent_t *) &aSG_ooa_event_pool[ 0 ];
  for ( i = 0; i < ${max_ooa_events.result}; i++ )
  {
    aSG_ooa_event_pool[ i ].mc_event_base.next = (${prefix.type}OoaEvent_t *) &(aSG_ooa_event_pool[ i + 1 ]);
  .invoke event_prioritization_needed = GetSystemEventPrioritizationNeeded()
  .if ( event_prioritization_needed.result )
    aSG_ooa_event_pool[ i ].mc_event_base.prev = (${prefix.type}OoaEvent_t *) 0;
  .end if .// prioritization needed
  }
  aSG_ooa_event_pool[ ${max_ooa_events.result} - 1 ].mc_event_base.next = (${prefix.type}OoaEvent_t *) 0;
}

  .invoke allocate_event_method = GetAllocateEventMethodName()
/*****************************************************************************
 * ${allocate_event_method.result}
 * Note:  event_size unused, reserved for future use, eliminated for now.
 ****************************************************************************/
${prefix.type}OoaEvent_t *
${allocate_event_method.result}()
{
  ${prefix.type}OoaEvent_t * event = (${prefix.type}OoaEvent_t *) 0;
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  if ( ! pSG_free_event_list )
  {
    ${prefix.symbolsw}UserEventFreeListEmptyCallout(); /* Bad news!  No more events.         */
  }
  else
  {
    event = pSG_free_event_list;       /* Grab front of the free list.     */
    pSG_free_event_list = event->next; /* Unlink from front of free list.  */
    ${prefix.define_csw}ClearOoaEventFlags( event );
  }
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  return event;
}

  .invoke new_event_method = GetNewEventName();
/*****************************************************************************
 * ${new_event_method.result}
 * Allocate the event and initialize the base attributes.
 ****************************************************************************/
${prefix.type}OoaEvent_t *
${new_event_method.result}( const void * const destination,
                            const ${prefix.type}OoaEventConstant_t * const event_info )
{
  ${prefix.type}OoaEvent_t * event = ${allocate_event_method.result}();
  ${prefix.define_csw}SetEventTargetInstance( event, destination );
  ${prefix.define_csw}SetEventDestDomainNumber( event, event_info->destination_domain_number );
  ${prefix.define_csw}SetEventDestObjectNumber( event, event_info->destination_object_number );
  ${prefix.define_csw}SetOoaEventNumber( event, event_info->event_number );
  ${prefix.define_csw}SetOoaEventFlags( event, event_info->event_flags );
  .if ( event_prioritization_needed.result )
  ${prefix.define_csw}SetOoaEventPriority( event, event_info->priority );
  .end if .// event_prioritization_needed
  return event;
}

  .invoke modify_event_method = GetModifyEventMethodName();
/*****************************************************************************
 * ${modify_event_method.result}
 * Update the base attributes of an event for polymorphic processing.
 ****************************************************************************/
${prefix.type}OoaEvent_t *
${modify_event_method.result}( ${prefix.type}OoaEvent_t * event,
                            const ${prefix.type}OoaEventConstant_t * const event_info )
{
  ${prefix.define_csw}SetEventDestDomainNumber( event, event_info->destination_domain_number );
  ${prefix.define_csw}SetEventDestObjectNumber( event, event_info->destination_object_number );
  ${prefix.define_csw}SetOoaEventNumber( event, event_info->event_number );
  ${prefix.define_csw}SetOoaEventFlags( event, event_info->event_flags );
  .if ( event_prioritization_needed.result )
  ${prefix.define_csw}SetOoaEventPriority( event, event_info->priority );
  .end if .// event_prioritization_needed
  return event;
}

  .invoke delete_event_method = GetDeleteEventMethodName()
/*****************************************************************************
 * ${delete_event_method.result}
 ****************************************************************************/
void
${delete_event_method.result}( void * event )
{
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  if ( event != 0 )
  {
    ((${prefix.type}OoaEvent_t *) event)->next = pSG_free_event_list;
    pSG_free_event_list = (${prefix.type}OoaEvent_t *) event;
  }
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
}

  .//
.end function
.//
.//============================================================================
.// CreateSystemOoaNonSelfEventQueue
.// Note: One extra container needed to terminate queue.
.//============================================================================
.function CreateSystemOoaNonSelfEventQueue
  .//
  .invoke max_non_self_events = GetMaxNonSelfEventsDefineName()
  .invoke event_prioritization_needed = GetSystemEventPrioritizationNeeded()
  .invoke prefix = GetPrefix()

static OoaEventQueue_s sSG_non_self_event_queue;

/*****************************************************************************
 * Zero the anchor points for the non- self-directed event queue.
 ****************************************************************************/
void
${prefix.symbol}InitializeOoaNonSelfEventQueue()
{
  sSG_non_self_event_queue.head = (${prefix.type}OoaEvent_t *) 0;
  sSG_non_self_event_queue.tail = (${prefix.type}OoaEvent_t *) 0;
}

/*****************************************************************************
 * Send an event to the instance queue.  Use priority where applicable.
 *
 * The following table summarizes the construction of events as
 * expected in Escher.  For each type of object (or pseudo-object)
 * that may be the source or target of an event, the expected
 * value of the source and destination handles in the event are
 * given.
 *
 *   object type | as event source     | as destination
 *   ------------+---------------------+----------------------
 *   instance    | handle              | handle
 *   assigner    | manufactured handle | 0
 *   creator     | manufactured handle | 0
 *   init        | manufactured handle | cannot be destination
 *   FBO         | manufactured handle | cannot be destination
 *
 ****************************************************************************/
  .invoke sendevent = GetSendEventMethodName()
void
${sendevent.result}( ${prefix.type}OoaEvent_t * event )
{
  .//
  .if ( event_prioritization_needed.result )
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  /* Insert event into empty event queue.  */
  if ( sSG_non_self_event_queue.tail == (${prefix.type}OoaEvent_t *) 0 )
  {
    sSG_non_self_event_queue.head = event;    /* first in queue */
    sSG_non_self_event_queue.tail = event;    /* last in queue */
    event->next = (${prefix.type}OoaEvent_t *) 0;
    event->prev = (${prefix.type}OoaEvent_t *) 0;
  }
  else
  /* No priority, append to end of queue.  */
  if ( event->priority == 0 )
  {
    event->next = (${prefix.type}OoaEvent_t *) 0;           /* input at tail end */
    event->prev = sSG_non_self_event_queue.tail;
    sSG_non_self_event_queue.tail->next = event; /* non empty */
    sSG_non_self_event_queue.tail = event;    /* tail points to last */
  }
  else
  /* Priority event, insert at proper position.  */
  {
    ${prefix.type}OoaEvent_t * e = sSG_non_self_event_queue.tail;
    /* Find slot before which to insert our new arrival.     */
    /* Break from the loop before the event we must follow.  */
    while ( e != (${prefix.type}OoaEvent_t *) 0 )
    {
      if ( event->priority <= e->priority )
        break;
      if ( ${prefix.define_csw}GetEventTargetInstance( event ) == ${prefix.define_csw}GetEventTargetInstance(e) ) {
        if ( ${prefix.define_csw}GetEventSendingInstance( event ) == ${prefix.define_csw}GetEventSendingInstance(e) ) {
          if ( ${prefix.define_csw}GetEventTargetInstance( event ) != 0 )
            break;
          else
            /* Creator or assigner with no target instance handle.  */
            /* We must interrogate the object and domain numbers.   */
            if ( ${prefix.define_csw}GetEventDestObjectNumber( event ) == ${prefix.define_csw}GetEventDestObjectNumber( e ) )
              if ( ${prefix.define_csw}GetEventDestDomainNumber( event ) == ${prefix.define_csw}GetEventDestDomainNumber( e ) )
                break;
        }
      }
      e = e->prev;
    }
    if ( e == (${prefix.type}OoaEvent_t *) 0 )
    {
      /* Slot not found, insert after head.  */
      e = sSG_non_self_event_queue.head;
      event->prev = (${prefix.type}OoaEvent_t *) 0;
      event->next = e;
      e->prev = event;
      sSG_non_self_event_queue.head = event;
    }
    else
    {
      /* Slot found, insert before.  */
      event->prev = e;
      event->next = e->next;
      e->next = event;
      if ( event->next != (${prefix.type}OoaEvent_t *) 0 )
        event->next->prev = event;
      if ( sSG_non_self_event_queue.tail == e )
        sSG_non_self_event_queue.tail = event;
    }
  }
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  .invoke send_dispatch_request = WakeUpMainDispatcher()
${send_dispatch_request.body}\
  .else
  event->next = (${prefix.type}OoaEvent_t *) 0;
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  /* Append the event to the tail end of the queue.  */
  /* No need to maintain prev pointers when not prioritizing.  */
  if ( sSG_non_self_event_queue.tail == (${prefix.type}OoaEvent_t *) 0 )
  {
    sSG_non_self_event_queue.head = event;
  }
  else
  {
    sSG_non_self_event_queue.tail->next = event;
  }
  sSG_non_self_event_queue.tail = event;
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  .invoke send_dispatch_request = WakeUpMainDispatcher()
${send_dispatch_request.body}\
  .end if .// prioritization needed
}

/*****************************************************************************
 * Drag an event from the instance directed queue if there is one.  This
 * routine also serves as the IsQueueEmpty routine.  A null return code 
 * indicates that the queue is empty.  Otherwise the handle to the
 * event will be returned.
 ****************************************************************************/
${prefix.type}OoaEvent_t *
${prefix.symbol}DequeueOoaNonSelfEvent()
{
  ${prefix.type}OoaEvent_t * event;
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  /* Assign the event from the head of the queue.  */
  event = sSG_non_self_event_queue.head;
  /* If the list is not empty, bump the head.  */
  if ( event != (${prefix.type}OoaEvent_t *) 0 )
  {
    sSG_non_self_event_queue.head = event->next;
    /* If empty, nullify tail.  Link prev pointers (if needed).  */
    if ( sSG_non_self_event_queue.head == (${prefix.type}OoaEvent_t *) 0 )
    {
      sSG_non_self_event_queue.tail = (${prefix.type}OoaEvent_t *) 0;
    }
  .if ( event_prioritization_needed.result )
    else
    {
      sSG_non_self_event_queue.head->prev = (${prefix.type}OoaEvent_t *) 0;
    }
  .end if .// prioritization needed
  }
  else
  {
    ${prefix.symbolsw}UserNonSelfEventQueueEmptyCallout();
  }
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  return event;
}

.end function
.//
.//============================================================================
.// CreateSystemOoaSelfEventQueue
.// Note: One extra container needed to terminate queue.
.//============================================================================
.function CreateSystemOoaSelfEventQueue
  .//
  .invoke max_self_events = GetMaxSelfEventsDefineName()
  .invoke prefix = GetPrefix()

/*****************************************************************************
 ****************************************************************************/
static OoaEventQueue_s sSG_self_event_queue;

/*****************************************************************************
 * Zero the anchor points for the self-directed event queue.
 ****************************************************************************/
void
${prefix.symbol}InitializeOoaSelfEventQueue()
{
  sSG_self_event_queue.head = (${prefix.type}OoaEvent_t *) 0;
  sSG_self_event_queue.tail = (${prefix.type}OoaEvent_t *) 0;
}

/*****************************************************************************
 * Send an event to the self queue.  No prioritization occurs on
 * this queue.
 ****************************************************************************/
void
${prefix.symbolsw}SendSelfEvent( ${prefix.type}OoaEvent_t * event )
{
  event->next = (${prefix.type}OoaEvent_t *) 0;
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  /* Append the event to the tail end of the queue.  */
  /* No need to maintain prev pointers for self directed events.  */
  if ( sSG_self_event_queue.tail == (${prefix.type}OoaEvent_t *) 0 )
  {
    sSG_self_event_queue.head = event;
  }
  else
  {
    sSG_self_event_queue.tail->next = event;
  }
  sSG_self_event_queue.tail = event;
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  .invoke send_dispatch_request = WakeUpMainDispatcher()
${send_dispatch_request.body}\
  .invoke debug_selfevent = DebugSendSelfEvent( "event" )
${debug_selfevent.body}
}

/*****************************************************************************
 * Drag an event from the self queue if there is one.  This routine
 * also serves as the IsQueueEmpty routine.  A null return code 
 * indicates that the queue is empty.  Otherwise the handle to the
 * event will be returned.
 ****************************************************************************/
${prefix.type}OoaEvent_t *
${prefix.symbol}DequeueOoaSelfEvent()
{
  ${prefix.type}OoaEvent_t * event;
  .invoke enter_mutex = EnterEventMutex()
  ${enter_mutex.result}
  /* Assign the event from the head of the queue.  */
  event = sSG_self_event_queue.head;
  /* If the list is not empty, bump the head.  */
  if ( event != (${prefix.type}OoaEvent_t *) 0 )
  {
    sSG_self_event_queue.head = event->next;               /* bump */
    /* If empty, nullify tail.  No need to maintain prev pointers
       for the self queue.  They are not used.  */
    if ( sSG_self_event_queue.head == (${prefix.type}OoaEvent_t *) 0 )
      sSG_self_event_queue.tail = (${prefix.type}OoaEvent_t *) 0;
  }
  else
  {
    ${prefix.symbolsw}UserSelfEventQueueEmptyCallout();
  }
  .invoke leave_mutex = LeaveEventMutex()
  ${leave_mutex.result}
  return event;
}

.end function
.//
.//============================================================================
.// Main Body:
.// Note: Order of rendering important for visibility of static data elements.
.//============================================================================
.clear
.//
.invoke hdr_file_ext = GetHeaderFileExtension()
.invoke src_file_ext = GetSourceFileExtension()
.//
.invoke prologue = CreateSystemEventPoolPrologue( FALSE )
.invoke epilogue = CreateSystemEventPoolFileEpilogue( FALSE )
.//
.invoke non_self_event_queue_needed = GetSystemNonSelfEventQueueNeeded()
.invoke self_event_queue_needed = GetSystemSelfEventQueueNeeded()
.//
${prologue.body}\
.//
.// *** OOA event pool.
.if ( non_self_event_queue_needed.result or self_event_queue_needed.result )
  .invoke ooa_event_pool = CreateSystemOoaEventPool( FALSE )
${ooa_event_pool.body}\
.end if
.//
.// *** OOA non-self directed event queue.
.if ( non_self_event_queue_needed.result )
  .invoke ooa_event_queue = CreateSystemOoaNonSelfEventQueue()
${ooa_event_queue.body}\
.end if
.//
.// *** OOA self directed event queue.
.if ( self_event_queue_needed.result )
  .invoke ooa_self_event_queue = CreateSystemOoaSelfEventQueue()
${ooa_self_event_queue.body}\
.end if
.//
${epilogue.body}
.//
.invoke file_name = GetSystemEventPoolFileName()
.invoke file_path = GetSystemLevelSourcePath()
.emit to file "${file_path.result}/${file_name.result}.${src_file_ext.result}"
.//
.//============================================================================
.//============================================================================
