.//============================================================================
.// File: system.arc
.//
.// Description:
.// System level coloring and dispacther level implementation code generation.
.//
.// Notice:
.// (C) Copyright 1998-2002 ROX Software, Inc.
.//     All rights reserved.
.// Enhancements provided by TOYO Corporation.
.// Modified for TOPPERS by RICOH CO,LTD. CC Development Section.
.//
.// This document contains confidential and proprietary information and
.// property of ROX Software, Inc.  No part of this document may be
.// reproduced without the express written permission of ROX Software, Inc.
.//============================================================================
.//
.invoke arc_env = GET_ENV_VAR( "ROX_MC_ARC_DIR" )
.assign mc_archetypes = arc_env.result
.if ( mc_archetypes == "" )
  .print "\nERROR: Environment variable ROX_MC_ARC_DIR not set."
  .exit 100
.end if
.//
.include "${mc_archetypes}/names.arc"
.include "${mc_archetypes}/files.arc"
.include "${mc_archetypes}/types.arc"
.include "${mc_archetypes}/sys_util.arc"
.include "${mc_archetypes}/clr_system2.arc"
.include "${mc_archetypes}/debug.arc"
.// Include system level user defined archetype functions.
.invoke system_colors_path = GetSystemLevelColoringFilesPath()
.invoke system_functions_file = GetSystemLevelUserFunctionsFile()
.include "${system_colors_path.result}/${system_functions_file.result}"
.//
.//============================================================================
.// CreateApplicationMain
.//
.//============================================================================
.function CreateApplicationMain
  .//
  .invoke sys_init_method = GetSystemInitializationMethodName()
  .invoke app_init_method = GetApplicationInitializationMethodName()
  .invoke sys_fini_method = GetSystemShutdownMethodName()
  .invoke app_fini_method = GetApplicationShutdownMethodName()
  .invoke event_base_name = GetOoaEventBaseClassName()
  .invoke delete_evt_method = GetDeleteEventMethodName()
  .invoke iconv_resources = GetIconvResources()
  .invoke prefix = GetPrefix()

  .//
extern void RunApplicationInitSequence( void );

${iconv_resources.var_declaration}\
bool ${prefix.method}run_flag = true;
  .// Define self reference storage and method.
  .invoke event_prioritization_needed = GetSystemEventPrioritizationNeeded()
  .if ( event_prioritization_needed.result )
static ${prefix.type}InstanceHandle_t ${prefix.result}self;
  .end if  .// prioritization_needed

/*****************************************************************************
 * main process
 ****************************************************************************/
  .invoke main_decl = GetMainTaskEntryDeclaration()
${main_decl.body}\
{
  ${event_base_name.result} * event;
${iconv_resources.initializer}\
  /* Execute user supplied system initialization. */
  ${prefix.symbolsw}UserInitializationCallout();

  /* Execute generated system initialization phase. */
  ${sys_init_method.result}();
  .//
  .// Generate call to debug monitor initialization if debugging.
  .invoke debug_monitor = DebugMonitorInit()
${debug_monitor.body}
  .//
  /* Execute OOA domain specific initialization phase(s). */
  ${app_init_method.result}();

  /* Execute application analysis initialization phase(s). */
  ${prefix.symbolsw}UserPreOoaInitializationCallout();   /* User supplied pre-init.  */
  RunApplicationInitSequence();        /* OOA init object states.  */
  ${prefix.symbolsw}UserPostOoaInitializationCallout();  /* User supplied post-init. */

  /* Start consuming events and dispatching background processes. */
  while ( ${prefix.method}run_flag )
  {
  .//
  .// *** Self directed OOA event queue processing.
  .invoke self_queue = GetSystemSelfEventQueueNeeded()
  .if ( self_queue.result )
    /* Dispatch an OOA self directed event.  */
    if ( ( event = ${prefix.symbol}DequeueOoaSelfEvent() ) != 0 )
    {
    .// Set up self reference for use by prioritized events (and others).
    .if ( event_prioritization_needed.result )
      /* Set up self reference for use by prioritized events (and others).  */
      ${prefix.result}self = (${prefix.type}InstanceHandle_t) ${prefix.define_csw}GetEventTargetInstance( event );
    .end if  .// prioritization_needed
      ( *DomainDispatcherTable[ ${prefix.define_csw}GetEventDestDomainNumber( event ) ] )( event );
      if ( ! ${prefix.define_csw}GetIsPolyEventInFlight( event ) )
        ${delete_evt_method.result}( event );
     .invoke send_dispatch_request = WakeUpMainDispatcher()
     ${send_dispatch_request.body}\
    }  /* end self */
    else
  .end if  .// self_queue.result
  .//
  .// *** Non-self directed OOA event queue processing.
  .invoke non_self_queue = GetSystemNonSelfEventQueueNeeded()
  .if ( non_self_queue.result )
    /* Dispatch one OOA non-self directed event.  */
    if ( ( event = ${prefix.symbol}DequeueOoaNonSelfEvent() ) != 0 )
    {
    .// Set up self reference for use by prioritized events (and others).
    .if ( event_prioritization_needed.result )
      /* Set up self reference for use by prioritized events (and others).  */
      ${prefix.result}self = (${prefix.type}InstanceHandle_t) ${prefix.define_csw}GetEventTargetInstance( event );
    .end if  .// prioritization_needed
      ( *DomainDispatcherTable[ ${prefix.define_csw}GetEventDestDomainNumber( event ) ] )( event );
      if ( ${prefix.define_csw}GetFsmReleasesEvent( event ) && ! ${prefix.define_csw}GetIsPolyEventInFlight( event ) )
        ${delete_evt_method.result}( event );
     .invoke send_dispatch_request = WakeUpMainDispatcher()
     ${send_dispatch_request.body}\
    }  /* end non-self */
  .else
    {} /* nop for potential empty else */
  .end if  .// non_self_queue.result
    .if ( event_prioritization_needed.result )
    /* Clear self when not in normal state action.    */
    ${prefix.result}self = (${prefix.type}InstanceHandle_t) 0;
    .end if  .// prioritization_needed
    /* If no event, sleep this task. */
  .invoke main_sleep = GetMainTaskSleepRequest()
${main_sleep.body}\
  .select any system from instances of TE_SYS where ( selected.MaxInterleavedBridges > 0 )
  .if ( not_empty system )
    .invoke dispatch_ileave_method = GetDispatchInterleaveBridgeMethodName()

    /* Launch (interrupt) bridge actions that occurred during state.  */
    ${dispatch_ileave_method.result}();
  .end if  .// not_empty system
  .//
  .// Generate call to debug monitor receive if debugging.
  .invoke debug_monitor = DebugMonitorReceive()
${debug_monitor.body}
  .//
    /* User supplied background processing callout. */
    ${prefix.symbolsw}UserBackgroundProcessingCallout();
  }

  /* Shutdown phasing */
  ${prefix.symbolsw}UserPreShutdownCallout();
  ${app_fini_method.result}();
  .//
  .// Generate call to debug monitor shutdown if debugging.
  .invoke debug_monitor = DebugMonitorShutdown()
${debug_monitor.body}
${iconv_resources.cleanup}\
  ${sys_fini_method.result}();
  ${prefix.symbolsw}UserPostShutdownCallout();

  .invoke return_body = GetMainTaskEntryReturn()
${return_body.body}\
}
  .//
.end function
.//
.//============================================================================
.// ResetSystemColoringParameters (private helper)
.//============================================================================
.function ResetSystemColoringParameters
  .//
  .select any system from instances of TE_SYS
  .assign system.ExecutableName = "rox.out"
  .assign system.MaxStringLen = 32
  .assign system.MaxObjExtent = 0
  .assign system.MaxRelExtent = 0
  .assign system.MaxSelectExtent = 0
  .assign system.TotalContainers = 0
  .assign system.MaxSelfEvents = 0
  .assign system.MaxNonSelfEvents = 0
  .assign system.MaxTimers = 0
  .assign system.MaxInterleavedBridges = 0
  .assign system.CollectionsFlavor = 0
  .assign system.SystemPrefix = "Escher_"
  .assign system.SystemMethodPrefix = "Escher_"
  .assign system.SystemTypePrefix = "Escher_"
  .assign system.SystemDefinePrefix = "Escher_"
  .assign system.SystemFilePrefix = ""
  .assign system.BppModelDebugger = FALSE
  .assign system.BppMaxStringLenDummy = 32
  .assign system.BppCollectionsFlavor = FALSE
  .//
  .select many queue_set from instances of TE_QUEUE
  .for each queue in queue_set
    .assign queue.RenderCode = FALSE
    .assign queue.MaxDepth = 0
    .assign queue.Blocking = FALSE
    .assign queue.PriorityLevel = 0
    .assign queue.InstEnabled = FALSE
  .end for
  .//
.end function
.//
.//============================================================================
.// CreateSystemFilePrologue
.//
.//============================================================================
.function CreateSystemFilePrologue
  .param boolean  gen_declaration
  .//
  .invoke file_name = GetSystemInitFileName()
/*****************************************************************************
  .if ( gen_declaration )
    .invoke file_ext = GetHeaderFileExtension()
 * File:  ${file_name.result}.${file_ext.result}
  .else
    .invoke file_ext = GetSourceFileExtension()
 * File:  ${file_name.result}.${file_ext.result}
  .end if
 *
 * Description:
 * Main entrance and parameters of the wild thing...
  .//
  .invoke copyright = GetCopyrightNotice()
${copyright.body}\
  .assign copyright.body = ""
 ****************************************************************************/

  .invoke file_ext = GetHeaderFileExtension()
  .if ( gen_declaration )
    .// *** Declarations section
    .//
    .invoke system_paramters = RenderSystemLimitsDeclarations()
    .//
    .invoke file_ext = GetHeaderFileExtension()
#ifndef $u{file_name.result}_$u{file_ext.result}
#define $u{file_name.result}_$u{file_ext.result}

#ifdef	__cplusplus
extern "C" {
#endif

${system_paramters.body}

  .else
    .// *** Definitions section
    .//
    .invoke file_name = GetSystemInitFileName()
#include "${file_name.result}.${file_ext.result}"
    .// *** Mechanism level includes
    .invoke generics_file = GetSystemMechanismsFileName()
    .invoke event_base_file = GetOoaEventBaseClassFileName()
    .invoke sets_file = GetSetOperationsFileName()
    .invoke user_file = GetSystemUserCalloutFileName()
#include "${generics_file.result}.${file_ext.result}"
#include "${event_base_file.result}.${file_ext.result}"
#include "${sets_file.result}.${file_ext.result}"
#include "${user_file.result}.${file_ext.result}"
    .invoke debugging = SystemModelDebuggingOn()
    .if ( debugging.result )
      .invoke ifdef = GetIfDefTM_Debug()
      .invoke endif = GetEndIfTM_Debug()
      .invoke layer_file = GetTMLayerFileName()
      .invoke iconv_resources = GetIconvResources()
${ifdef.result}
#include "${layer_file.result}.${file_ext.result}"
#include ${iconv_resources.header_file}
${endif.result}
    .end if .// debugging.result
    .//
    .select many domain_set from instances of TE_DLSP
    .for each domain in domain_set
      .invoke file_name = GetDomainInitFileNameFromString( domain.RegisteredDomName )
#include "${file_name.result}.${file_ext.result}"
    .end for
    .// Generate call to debug monitor initialization if debugging.
    .invoke debug_monitor = DebugMonitorInclude()
${debug_monitor.body}

  .end if
.end function
.//
.//============================================================================
.// CreateSystemFileEpilogue
.//
.//============================================================================
.function CreateSystemFileEpilogue
  .param boolean  gen_declaration
  .//
  .if ( gen_declaration )
    .// *** Declarations section
    .invoke file_name = GetSystemInitFileName()
    .invoke file_ext = GetHeaderFileExtension()

#ifdef	__cplusplus
}
#endif

#endif  /* $u{file_name.result}_$u{file_ext.result} */
  .else
    .// *** Definitions section
  .end if
.end function
.//
.//============================================================================
.// CreateSystemInitialization
.//
.// Generates the application header file
.//============================================================================
.function CreateSystemInitialization
  .param boolean  gen_declaration
  .//
  .invoke sys_init_method = GetSystemInitializationMethodName()
  .invoke app_init_method = GetApplicationInitializationMethodName()
  .invoke event_prioritization_needed = GetSystemEventPrioritizationNeeded()
  .invoke prefix = GetPrefix()
  .invoke globalself = GetGlobalSelf()
  .//
  .if ( gen_declaration )
    .// Declare method for access to self.
    .if ( event_prioritization_needed.result )
extern void * ${globalself.result}( void );
    .end if  .// prioritization_needed
extern void ${sys_init_method.result}( void );
extern void ${app_init_method.result}( void );
  .else
    .// Define method for access to self.
    .if ( event_prioritization_needed.result )

/*****************************************************************************
 * ${globalself.result}:  Access the reference to event target instance.
 ****************************************************************************/
void *
${globalself.result}()
{
  return (void *) ${prefix.result}self;
}
    .end if  .// prioritization_needed
 
/*****************************************************************************
 * ${sys_init_method.result}
 ****************************************************************************/
void
${sys_init_method.result}()
{
  /* Initialize the container pool. */
  .invoke factory_init = GetCollectionFactoryInitMethodName()
  ${factory_init.result}();

    .invoke non_self_event_queue_needed = GetSystemNonSelfEventQueueNeeded()
    .invoke self_event_queue_needed = GetSystemSelfEventQueueNeeded()
    .//
    .if ( non_self_event_queue_needed.result or self_event_queue_needed.result )
  /* Initialize OOA event pool and queues. */
  ${prefix.symbol}InitializeOoaEventPool();
    .end if
    .//
    .if ( non_self_event_queue_needed.result )
  ${prefix.symbol}InitializeOoaNonSelfEventQueue();
    .end if
    .//
    .if ( self_event_queue_needed.result )
  ${prefix.symbol}InitializeOoaSelfEventQueue();
    .end if
}

/*****************************************************************************
 * ${app_init_method.result}
 ****************************************************************************/
void
${app_init_method.result}()
{
   /* Initialize all the domains */
    .select many domain_set from instances of TE_DLSP
    .for each domain in domain_set
      .invoke init_method = GetDomainInitMethodNameFromString( domain.RegisteredDomName )
  ${init_method.result}();
    .end for
}
  .end if  .// gen_declaration
  .//
.end function
.//
.//============================================================================
.// CreateSystemShutdown
.//
.//============================================================================
.function CreateSystemShutdown
  .param boolean  gen_declaration
  .//
  .invoke sys_fini_method = GetSystemShutdownMethodName()
  .invoke app_fini_method = GetApplicationShutdownMethodName()
  .//
  .if ( gen_declaration )
extern void ${sys_fini_method.result}( void );
extern void ${app_fini_method.result}( void );
  .else
    .//

/*****************************************************************************
 * ${sys_fini_method.result}
 ****************************************************************************/
void
${sys_fini_method.result}()
{}

/*****************************************************************************
 * ${app_fini_method.result}
 ****************************************************************************/
void
${app_fini_method.result}()
{
   /* Shutdown all the domains */
  .select many domain_set from instances of TE_DLSP
  .for each domain in domain_set
    .invoke shutdown_method = GetDomainFiniMethodNameFromString( domain.RegisteredDomName )
  ${shutdown_method.result}();
  .end for
}

  .end if  .// gen_declaration
  .//
.end function
.//
.//============================================================================
.// CreateOoaDomainDispatcherTable
.//
.//============================================================================
.function CreateOoaDomainDispatcherTable
  .//
  .select many dom_set from instances of TE_DLSP
  .assign num_ooa_doms = cardinality dom_set
  .assign dom_count = 0
  .assign table_size = num_ooa_doms + 1
  .invoke dispatch = GetDomainDispatcherNameFromString( "" )
  .//
/*****************************************************************************
 * system-level domain dispatcher table
 * This table contains pointers to dispatcher tables for each of
 * the OOA domains generated for this system.
 ****************************************************************************/
static const ${dispatch.type} DomainDispatcherTable[ ${table_size} ] =
{
  0,
  .//
  .for each dom in dom_set
    .invoke dispatch_method = GetDomainDispatcherNameFromString( dom.RegisteredDomName )
  (${dispatch_method.type}) ${dispatch_method.result}\
    .assign dom_count = dom_count + 1
    .if ( dom_count != num_ooa_doms )
,
    .else

    .end if
  .end for
};

.end function
.//
.//============================================================================
.// CreateInterleaveBridge
.// Generate declarations and C source for the post and dispatch
.// routines for the interleaved bridges.
.//============================================================================
.function CreateInterleaveBridge
  .param boolean  gen_declaration
  .//
  .select any system from instances of TE_SYS where ( selected.MaxInterleavedBridges > 0 )
  .if ( not_empty system )
    .//
    .invoke ileave_method = GetInterleaveBridgeMethodName()
    .invoke dispatch_ileave_method = GetDispatchInterleaveBridgeMethodName()
    .invoke get_ilb_data = GetGetInterleaveBridgeDataMethodName()
    .invoke prefix = GetPrefix()
    .//
    .if ( gen_declaration )
extern void * ${ileave_method.result}( void (vfp)(void) );
extern void ${dispatch_ileave_method.result}( void );
extern void * ${get_ilb_data.result}( void );
    .else
      .invoke ileave_define_name = GetMaxInterleavedBridgesDefineName()
      .invoke ileave_data_define_name = GetMaxInterleavedDataDefineName()
typedef void ( * interleaved_bridge_t )( void );
static interleaved_bridge_t interleaved_bridges[ ${ileave_define_name.result} ];
typedef struct { char space[ ${ileave_data_define_name.result} ]; } ilb_data_t;
static ilb_data_t ilb_data[ ${ileave_define_name.result} ];
static unsigned char interleaved_bridges_tail = 0;
static unsigned char current_interleaved_bridge = 0;
/*****************************************************************************
 * ${ileave_method.result}
 * Post a bridge routine for execution after any executing state
 * action completes.
 * Protect the data structures from being clobbered by another context
 * by disabling interrupts around the data access.
 * Return a pointer to the argument data used by this bridge operation.
 ****************************************************************************/
void *
${ileave_method.result}( void (vfp)(void) )
{
  void * ilb = (void *) 0;
  .invoke disable_interrupts = UserDisableInterrupts()
  ${disable_interrupts.result}
  if ( interleaved_bridges_tail < ${ileave_define_name.result} ) 
  {
    ilb = (void *) &ilb_data[ interleaved_bridges_tail ];
    interleaved_bridges[ interleaved_bridges_tail++ ] = vfp;
  } else {
    /* Overflowed array of interleaved bridges.  */
    ${prefix.symbol}UserInterleavedBridgeOverflowCallout();
  }
  .invoke enable_interrupts = UserEnableInterrupts()
  ${enable_interrupts.result}
  .invoke send_dispatch_request = WakeUpMainDispatcherInterleave()
${send_dispatch_request.body}\
  return ilb;
}

/*****************************************************************************
 * ${dispatch_ileave_method.result}
 * Sequentially execute each of the bridge routines that have been
 * posted during the most recent executing state action.
 * Protect the data structures from being clobbered by another context
 * by disabling interrupts around the data access.
 ****************************************************************************/
void
${dispatch_ileave_method.result}()
{
  ${disable_interrupts.result}
  current_interleaved_bridge = 0;
  while ( current_interleaved_bridge < interleaved_bridges_tail )
  {
    ${enable_interrupts.result}
    /* Bridge runs with interrupts enabled.  */
    interleaved_bridges[ current_interleaved_bridge ]();
    ${disable_interrupts.result}
    current_interleaved_bridge++;
  }
  interleaved_bridges_tail = 0;
  ${enable_interrupts.result}
}

/*****************************************************************************
 * Get a pointer to the data for the current interleaved bridge.
 ****************************************************************************/
void *
${get_ilb_data.result}()
{ return (void *) &ilb_data[ current_interleaved_bridge ]; }
    .end if  .// gen_declaration
  .end if  .// not_empty system
.end function
.//
.//============================================================================
.// RenderSystemLimitsDeclarations
.//============================================================================
.function RenderSystemLimitsDeclarations
  .//
  .invoke max_str_len_name = GetMaxStringLengthDefineName()
  .invoke max_containoids_name = GetMaxContainersDefineName()
  .invoke max_self_events_name = GetMaxSelfEventsDefineName()
  .invoke max_non_self_events_name = GetMaxNonSelfEventsDefineName()
  .invoke max_ooa_events_name = GetMaxOoaEventsDefineName()
  .invoke max_ooa_timers_name = GetMaxOoaTimersDefineName()
  .//
  .invoke max_obj_extent_name = GetTotalObjectExtentDefineName()
  .invoke max_rel_extent_name = GetTotalRelationshipExtentDefineName()
  .invoke max_tran_extent_name = GetTotalTransientExtentDefineName()
  .//
  .invoke max_interleaved_bridges_define_name = GetMaxInterleavedBridgesDefineName()
  .invoke max_interleaved_data_define_name = GetMaxInterleavedDataDefineName()
  .//
  .select any system from instances of TE_SYS
  .select many dom_dlsp_set related by system->TE_DLSP[R2002]
  .//
  .// container allocation
  .assign max_obj_extent = 0
  .assign max_rel_extent = 0
  .assign max_sel_extent = 0
  .// event allocation (and thus queue sizes)
  .assign max_self_events = 0
  .assign max_non_self_events = 0
  .assign max_timers = 0
  .// bridge statistics
  .assign interleaved_bridges = 0
  .//
  .for each dom_dlsp in dom_dlsp_set
    .assign max_obj_extent = max_obj_extent + dom_dlsp.MaxObjExtent
    .assign max_rel_extent = max_rel_extent + dom_dlsp.MaxRelExtent
    .assign max_sel_extent = max_sel_extent + dom_dlsp.MaxSelectExtent
    .assign max_self_events = max_self_events + dom_dlsp.MaxSelfEvents
    .assign max_non_self_events = max_non_self_events + dom_dlsp.MaxNonSelfEvents
    .assign max_timers = max_timers + dom_dlsp.MaxTimers
    .assign interleaved_bridges = interleaved_bridges + dom_dlsp.InterleavedBridges
  .end for  .// dom_dlsp in dom_dlsp_set

/*****************************************************************************
 * System Name:  ${system.ExecutableName}
 * System ID:    ${system.SystemID}
 *
 * Model Compiler Product Information:
 * Product:  ${system.ModelCompilerName}
 * Version:  ${system.ModelCompilerVersion}
 * S/N:      ${system.ModelCompilerSerNum}
.// * License:  ${system.ModelCompilerKey}
.// * BridgePoint License Key:  ${system.BridgePointLicenseKey}
 *
 * System default/colored values:
 * MaxStringLen:  ${system.MaxStringLen}
 * MaxObjExtent:  ${system.MaxObjExtent}
 * MaxRelExtent:  ${system.MaxRelExtent}
 * MaxSelectExtent:  ${system.MaxSelectExtent}
 * MaxSelfEvents:  ${system.MaxSelfEvents}
 * MaxNonSelfEvents:  ${system.MaxNonSelfEvents}
 * MaxTimers:  ${system.MaxTimers}
 * MaxInterleavedBridges:  ${system.MaxInterleavedBridges}
 * MaxInterleavedBridgeDataSize:  ${system.MaxInterleavedBridgeDataSize}
 * CollectionsFlavor:  ${system.CollectionsFlavor}
 ****************************************************************************/

#define ${max_str_len_name.result} ${system.MaxStringLen}

  .// *** Container pool requirements
  .if ( system.MaxObjExtent == 0 )
    .if ( max_obj_extent != 0 )
#define ${max_obj_extent_name.result} ${max_obj_extent}
    .else
#define ${max_obj_extent_name.result} 1
    .end if
  .else
    .if ( system.MaxObjExtent != 0 )
/* Note: User adjusted from calculated worst case value: ${max_obj_extent} */
#define ${max_obj_extent_name.result} ${system.MaxObjExtent}
    .else
#define ${max_obj_extent_name.result} 1
    .end if
  .end if

  .if ( system.MaxRelExtent == 0 )
#define ${max_rel_extent_name.result} ${max_rel_extent}
  .else
/* Note: User adjusted from calculated worst case value: ${max_rel_extent} */
#define ${max_rel_extent_name.result} ${system.MaxRelExtent}
    .assign max_rel_extent = system.MaxRelExtent
  .end if

  .if ( system.MaxSelectExtent == 0 )
#define ${max_tran_extent_name.result} ${max_sel_extent}
  .else
/* Note: User adjusted from calculated estimated value: ${max_sel_extent} */
#define ${max_tran_extent_name.result} ${system.MaxSelectExtent}
    .assign max_sel_extent = system.MaxSelectExtent
  .end if

/* As of version 2.2.11, we will not calculate the containers
 * for object extents.  Since we need exactly one container per
 * instance, we will allocate them one to one in the object .c files.
 */
#define ${max_containoids_name.result} ( ${max_rel_extent_name.result} + ${max_tran_extent_name.result} )
  .// Keep the number of containers allocated for use in mechanism gen.
  .assign system.TotalContainers = max_sel_extent + max_rel_extent

  .// *** Event pool/queue requirements
  .if ( system.MaxSelfEvents == 0 )
#define ${max_self_events_name.result} ${max_self_events}
  .else
/* Note:  User adjusted from calculated estimate value:  ${max_self_events}.  */
#define ${max_self_events_name.result} ${system.MaxSelfEvents}
  .end if

  .if ( system.MaxNonSelfEvents == 0 )
#define ${max_non_self_events_name.result} ${max_non_self_events}
  .else
/* Note:  User adjusted from calculated estimate value:  ${max_non_self_events}.  */
#define ${max_non_self_events_name.result} ${system.MaxNonSelfEvents}
  .end if

#define ${max_ooa_events_name.result} ( ${max_self_events_name.result} + ${max_non_self_events_name.result} )

  .if ( system.MaxTimers == 0 )
#define ${max_ooa_timers_name.result} ${max_timers}
  .else
/* Note: User adjusted from calculated estimate value:  ${max_timers}.  */
#define ${max_ooa_timers_name.result} ${system.MaxTimers}
  .end if
  .assign system.MaxTimers = max_timers

  .if ( system.MaxInterleavedBridges == 0 )
#define ${max_interleaved_bridges_define_name.result} ${interleaved_bridges}
    .// Store result of domain accumulation in system value to
    .// serve as flag for interleave bridges code generation.
    .assign system.MaxInterleavedBridges = interleaved_bridges
  .else
/* Note: User adjusted from calculated estimate value:  ${interleaved_bridges}.  */
#define ${max_interleaved_bridges_define_name.result} ${system.MaxInterleavedBridges}
  .end if

  .if ( system.MaxInterleavedBridgeDataSize == 0 )
    .// Use default if nothing touched in coloring.
#define ${max_interleaved_data_define_name.result} 8
    .assign system.MaxInterleavedBridgeDataSize = 8
  .else
    .// Use colored value.
#define ${max_interleaved_data_define_name.result} ${system.MaxInterleavedBridgeDataSize}
  .end if

  .for each dom_dlsp in dom_dlsp_set

/*****************************************************************************
 * Domain Name:          ${dom_dlsp.RegisteredDomName}
 * Domain ID:            ${dom_dlsp.RegisteredDomID}
 * Repository:           ${dom_dlsp.RepositoryName}
 * OOA Version:          ${dom_dlsp.RepositoryVersion}
 *
 * MaxObjExtent:         ${dom_dlsp.MaxObjExtent}
 * MaxRelExtent:         ${dom_dlsp.MaxRelExtent}
 * MaxSelectExtent:      ${dom_dlsp.MaxSelectExtent}
 * MaxSelfEvents:        ${dom_dlsp.MaxSelfEvents}
 * MaxNonSelfEvents:     ${dom_dlsp.MaxNonSelfEvents}
 * MaxPriorityEvents:    ${dom_dlsp.MaxPriorityEvents}
 * MaxTimers:            ${dom_dlsp.MaxTimers}
 * InterleavedBridges:   ${dom_dlsp.InterleavedBridges}
 * InterleavedDataSize:  ${system.MaxInterleavedBridgeDataSize}
 * CollectionsFlavor:    ${system.CollectionsFlavor}
 ****************************************************************************/
  .end for  .// dom_dlsp in dom_dlsp_set

.end function
.//
.//============================================================================
.// Main Body:
.//
.// Initialize the generator database with coloring information.
.// Note that the order of processing is important here!
.//============================================================================
.clear
.//
.invoke hdr_file_ext = GetHeaderFileExtension()
.invoke src_file_ext = GetSourceFileExtension()
.//
.invoke system_colors_path = GetSystemLevelColoringFilesPath()
.//
.assign fatal_errors = FALSE
.//
.// *** System level coloring
.invoke ResetSystemColoringParameters()
.invoke system_coloring_file = GetSystemColoringFileName()
.include "${system_colors_path.result}/${system_coloring_file.result}"
.print "\n  System level coloring complete."
.//
.// stop generating code for Pallete creating xml.
.invoke bpp_env = GET_ENV_VAR( "BPP_CREATE_XML" )
.assign bpp_create_xml = bpp_env.result
.if ( bpp_create_xml == "" )
.//
.// *** Schema updating (only after system level coloring!).
.invoke SetSystemSelfEventQueueParameters()
.invoke SetSystemNonSelfEventQueueParameters()
.//
.// *** Generate header file(s).
.//
.invoke prologue = CreateSystemFilePrologue( TRUE )
.invoke system_init = CreateSystemInitialization( TRUE )
.invoke epilogue = CreateSystemFileEpilogue( TRUE )
.invoke system_shutdown = CreateSystemShutdown( TRUE )
.invoke interleave_bridge_support = CreateInterleaveBridge( TRUE )
.invoke domain_ids = DeclareDomainIndentityEnums()
.//
${prologue.body}\
${domain_ids.body}
${system_init.body}\
${system_shutdown.body}\
${interleave_bridge_support.body}\
${epilogue.body}\
.//
.invoke file_name = GetSystemInitFileName()
.invoke directory = GetSystemLevelIncludePath()
.emit to file "${directory.result}/${file_name.result}.${hdr_file_ext.result}"
.//
.// *** Generate source file(s).
.//
.invoke prologue = CreateSystemFilePrologue( FALSE )
.invoke system_init = CreateSystemInitialization( FALSE )
.invoke system_shutdown = CreateSystemShutdown( FALSE )
.invoke interleave_bridge_support = CreateInterleaveBridge( FALSE )
.invoke program_main = CreateApplicationMain()
.invoke ooa_dom_dispath_table = CreateOoaDomainDispatcherTable()
.//
${prologue.body}\
${ooa_dom_dispath_table.body}\
${program_main.body}\
${system_init.body}\
${system_shutdown.body}\
${interleave_bridge_support.body}\
.//
.invoke file_name = GetSystemInitFileName()
.invoke file_path = GetSystemLevelSourcePath()
.emit to file "${file_path.result}/${file_name.result}.${src_file_ext.result}"
.//
.end if
.//============================================================================
.//============================================================================
